;************************************************************************************
; 
; This is a modified version of the original V1.1 bootloader stored in CH552 devices
; The functionallity of the loader is still the same as the original one
; While i did not find a way to replace the original loader at 0x3800 this one can
; be flashed on every page boundary.
;           
; Modifications:
; 1. changed the Version bcd field to 0x0200 so this version can be identified by usbview
; 2. link address can be changed to start at every page boundary
; 3. removed the Init code to make it more compact   
; 4. added a bugfix to usbcode (unstall when setup arrives) 
; 5. replace LCALL LJMP with ACALL AJMP 
;
;************************************************************************************

$nomod51
$nolist
$include (..\inc\ch552_keil.h)
$list
dseg at 0x08
wValueLo:          ds 1
reqLen:            ds 1       ;wlengthLow
pCall:             ds 3       ;Call address for exec a program
bRequest:          ds 1
pDesc:             ds 3
scramble:          ds 4       ;Scramble Key
Addr:              ds 2
rLen:              ds 1
RAM_18:            ds 3       ;used as overlay
RAM_1B:            ds 3       ;
RAM_1E:            ds 1       ;

bseg at 0
bResponsValid:     dbit 1     ;1 if there is a valid reponse packet availble
bWriteProtected:   dbit 1     ;1 if CodeFlash is write protected 
bRunMode:          dbit 1     ;1 if in exec mode
bSerialMode:       dbit 1     ;1 if in serial mode
bRequestError:     dbit 1     ;1 if a usb request is not supported

dseg at 0x21
cmdbuffer:         ds 64      ;EP Buffer size
Stack:

xseg at 0
EP0_BUFFER:        ds  8      ; 0x0000 the control endpoint
                   ds  4      ; spare
EP2_BUFFER:        ds 64      ; 0x000C bulk out
Response:          ds 64      ; 0x004C bulk in
ExtraBuffer:       ds 64      ; 0x008C

StartAddress      equ  0x3000

cseg at StartAddress          ; bootloader start
    ajmp  C_C51STARTUP

; =============== S U B R O U T I N E =======================================
Functiondispatcher:
;alias because this is overlay memory
_dispresult set RAM_18+0                ;dispatcher result
_respLen    set RAM_18+2                ;response length 
_j          set RAM_18+1                ;loopvar  
_i          set R1                      ;another loopvar
_FlAddrH    set RAM_1B+0
_FlAddrL    set RAM_1B+1

    clr   A
    mov   _dispresult, A                ;dispresult=0
    mov   _respLen, A                   ;respLen=0
    acall ToggleProgMode
    mov   R0, #cmdbuffer                ;switch cmdbuffer[0]
    mov   A, @R0
    acall _cc_case

    dw    FunctionA2                    ;Chipdetect_v1
    db    0xA2
    dw    FunctionA5                    ;RunCommand_V1
    db    0xA5
    dw    FunctionA6                    ;SetScrambleKey_V1
    db    0xA6
    dw    FunctionA7                    ;VerifyCodeFlash_V1
    db    0xA7
    dw    FunctionAA                    ;WriteCodeFlash_V1
    db    0xA8
    dw    FunctionA9                    ;ErasePage_V1
    db    0xA9
    dw    FunctionB5                    ;EraseDataFlash_V1 
    db    0xB5
    dw    FunctionB6                    ;WriteDataFlash_V1
    db    0xB6
    dw    FunctionB7                    ;ReadDataFlash_V1
    db    0xB7
    dw    FunctionB8                    ;WriteConfig_V1
    db    0xB8
    dw    FunctionB9                    ;ReadConfig_V1
    db    0xB9
    dw    FunctionBA                    ;WriteLoaderCfg_V1
    db    0xBA
    dw    FunctionBB                    ;GetChipId_V1
    db    0xBB
    dw    0
    dw    DispatcherDefault

;***************************************
;chipdetectV1
;<A2> <x> <"USB DBG CH559 & ISP">
FunctionA2:                             ;
    mov   _dispresult, CHIP_ID          ;
    clr   A
    mov   _i, A                         ;for (i=0,i<0x13;i++)
Loop1_A2:                               ;
    mov   A, _i
    mov   DPTR, #CopyRightMsg           ;  ->"USB DBG CH559 & ISP"
    movc  A, @A+DPTR
    mov   R7, A
    mov   A, #cmdbuffer+2               ;
    add   A, _i
    mov   R0, A
    mov   A, @R0
    xrl   A, R7
    jz    $+4
    ajmp  Dispatcher_error              ;
    inc   _i
    cjne  _i, #0x13, Loop1_A2
    ajmp  Dispatcher_break
;***************************************
;verify_V1
;<A7> <len> <addrL> <addrH> [<data>]
FunctionA7:                             ;
    mov   R0, #cmdbuffer+3              ;
    mov   A, @R0
    mov   R6, A
    dec   R0
    mov   A, @R0
    mov   R5, A
    mov   A, R5
    mov   Addr+0, R6                    ;addr
    mov   Addr+1, A
    clr   A                             ;j=0
    mov   _j, A                         ;
Loop1_A7:                               ;do
    clr   A                             ;{
    mov   _i, A                         ;   i=0;
Loop2_A7:                               ;   {
    mov   A, #scramble                  ;
    add   A, _i
    mov   R0, A
    mov   A, @R0
    mov   R7, A                         ;       R7 = scramble[i];
    mov   A, #cmdbuffer+4               ;
    add   A, _j
    mov   R0, A
    mov   A, @R0
    xrl   A, R7
    mov   R7, A                         ;       R7 = scramble[i] ^ cmdbuffer[4+j];
    mov   DPL, Addr+1                   ;
    mov   DPH, Addr+0                   ;
    clr   A
    movc  A, @A+DPTR                    ;       A = CBYTE[addr]
    xrl   A, R7                         ;
    jz    $+4
    ajmp  Dispatcher_error
    inc   _j                            ;       j ++
    inc   Addr+1
    mov   A, Addr+1                     ;       addr++
    jnz   $+4
    inc   Addr+0
    inc   _i                            ;       i++
    cjne  _i, #4, Loop2_A7              ;   }while (i<4)
    mov   A, _j
    clr   C
    mov   R0, #cmdbuffer+1              ;}while (j < cmdbuffer[1]);
    subb  A, @R0
    jc    Loop1_A7
    ajmp  Dispatcher_break
;***************************************
;WriteCodeFlash_V1
;<aa> <len> <addrl> <addrH> [<datal>,<dataH>]
FunctionAA:                             ;
    jnb   bWriteProtected, $+5
    ajmp  DispatcherDone
    mov   R0, #cmdbuffer+3              ;FlAddr = Cmdbuffer[4] << 8 |  Cmdbuffer[3]
    mov   A, @R0
    mov   R6, A
    dec   R0
    mov   A, @R0
    mov   R5, A
    mov   A, R6
    mov   _FlAddrH, A
    mov   A, R5
    mov   _FlAddrL, A
    clr   A                             ;for (j=0;j<..;j++) 
    mov   _j, A
Loop1_AA:                               ;{
    clr   A                             ;   for (i=0;i<..;i++)
    mov   _i, A
Loop2_AA:                               ;   {
    mov   A, _i
    add   A, ACC                        ;      R7 = scramble[i<<1+1]
    add   A, #scramble+1
    mov   R0, A
    mov   A, @R0
    mov   R7, A
    mov   A, #cmdbuffer+5               ;      A = cmdbuffer[5+i]
    add   A, _j
    mov   R0, A
    mov   A, @R0
    xrl   A, R7                        ;      R6 = A ^ R7
    mov   R6, A
    mov   A, _i
    add   A, ACC                       ;      R5 = scramble[i<<1]
    add   A, #scramble+0
    mov   R0, A
    mov   A, @R0
    mov   R5, A
    mov   A, #cmdbuffer+4              ;      A = cmdbuffer[4+i]
    add   A, _j
    mov   R0, A
    mov   A, @R0
    xrl   A, R5                        ;      R4 = R5 ^ A
    mov   R5, A
    mov   A, R6
    mov   R4, A
    mov   R6, #0
    mov   A, _FlAddrL                  ;      dispresult=FlashCodeEntry(addr+j,R4/R5)
    add   A, _j
    mov   R7, A
    mov   A, R6
    addc  A, _FlAddrH
    mov   R6, A
    acall FlashCodeEntry
    mov   _dispresult, R7
    mov   A, _dispresult               ;      if (dispresult) break;
    jz    $+4
    ajmp  Dispatcher_break             ;
    inc   _j                           ;      j+=2
    inc   _j
    inc   _i                           ;      i++
    cjne  _i, #2, Loop2_AA
    mov   A, _j
    clr   C
    mov   R0, #cmdbuffer+1             ; '"'
    subb  A, @R0
    jc    Loop1_AA
    ajmp  Dispatcher_break
;**************************************
;erase_page_cmd_v1
;<A9> <len> <pageL> <pageh>
FunctionA9:                ;
    mov   R0, #cmdbuffer+3              ;FlAddr = Cmdbuffer[4] << 8 |  Cmdbuffer[3]
    mov   A,  @R0
    mov   R6, A
    dec   R0
    mov   A,  @R0
    mov   R5, A
    mov   A,  R6
    mov   _FlAddrH, A
    mov   A, R5
    mov   _FlAddrL, A
    jb    bWriteProtected, $+5          ;if (!bWriteProtected) break;
    ajmp  Dispatcher_break
    orl   A, _FlAddrH                  ;if (FlAddr)
    jz    code_3921
Dispatcher_error:                      ;{
    mov   _dispresult, #0xFF           ;   dispresult = -1; break;  
    ajmp  Dispatcher_break             ;}
code_3921:                             ;do
    mov   A, #0xFF
    mov   R5, A
    mov   R4, A
    mov   R7, _FlAddrL
    mov   R6, _FlAddrH
    acall FlashCodeEntry               ;   FlashCodeEntry(FlAddr,0xFFFF)   
    mov   _dispresult, R7
    mov   A, _dispresult
    jz    $+4
    ajmp  Dispatcher_break
    mov   A, #2                        ;   FlAddr+=2;
    add   A, _FlAddrL
    mov   _FlAddrL, A
    clr   A
    addc  A, _FlAddrH
    mov   _FlAddrH, A
    anl   A, #3
    orl   A, _FlAddrL
    jnz   code_3921                    ;} while(FlAddr & 0x3FF) 
    clr   bWriteProtected
    ajmp  Dispatcher_break
;***************************************
;setkeyV1
;<A6> <len> <key0> <key1> <key2> <key3>
FunctionA6:                ;
    mov   R0, #cmdbuffer+2             ; '#'
    mov   A, @R0
    mov   scramble+0, A
    inc   R0
    mov   A, @R0
    mov   scramble+1, A
    inc   R0
    mov   A, @R0
    mov   scramble+2, A
    inc   R0
    mov   A, @R0
    mov   scramble+3, A
    ajmp  Dispatcher_break
;***************************************
;write 2 config bytes
;<b8> <cfgl> <cfgh> <>
FunctionB8:                             ;
    mov   DPTR, #ROM_CFG_ADDR
    clr   A
    movc  A, @A+DPTR
    mov   R0, #cmdbuffer+2              ; '#'
    xrl   A, @R0
    jnz   code_3972
    inc   DPTR
    movc  A, @A+DPTR
    inc   R0
    xrl   A, @R0
    jnz   code_3972
    ajmp  Dispatcher_break

code_3972:                              ;^
    mov   R0, #cmdbuffer+3              ; '$'
    sjmp  code_39C4
;***************************************
;WriteLoaderCfg
FunctionBA:       ;
    mov   DPTR, #LoaderOptions+2        ;if  (cmdbuffer[4] != CBYTE[0x3FF6])
    clr   A
    movc  A, @A+DPTR
    mov   R0, #cmdbuffer+4              ;
    xrl   A, @R0
    jnz   code_3986                     ;  &&(cmdbuffer[5] != CBYTE[0x3FF7])
    inc   DPTR
    movc  A, @A+DPTR
    inc   R0
    xrl   A, @R0
    jz    code_39A6

code_3986:                              ;
    mov   R0, #cmdbuffer+3              ; '$'
    mov   R7, #low LoaderOptions        ; ''
    acall FlashCode
    mov   _dispresult, R7
    mov   A, _dispresult
    jz    $+4
    ajmp  Dispatcher_break

    mov   R0, #cmdbuffer+5              ; '&'
    mov   R7, #low LoaderOptions+2      ; ''
    acall FlashCode
    mov   _dispresult, R7
    mov   A, _dispresult
    jz    $+4
    ajmp  Dispatcher_break

code_39A6:                              ;
    mov   R0, #cmdbuffer+1              ; '"'
    mov   A, @R0
    xrl   A, #8
    jz    $+4
    ajmp  Dispatcher_break

    mov   DPTR, # ROM_CFG_ADDR
    movc  A, @A+DPTR
    mov   R0, #cmdbuffer+8              ; ')'
    xrl   A, @R0
    jnz   code_39C2
    inc   DPTR
    movc  A, @A+DPTR
    inc   R0
    xrl   A, @R0
    jnz   code_39C2
    ajmp  Dispatcher_break

code_39C2:        ;
    mov   R0, #cmdbuffer+9              ; '*'

code_39C4:        ;
    mov   A, @R0
    mov   R6, A
    dec   R0
    mov   A, @R0
    mov   R5, A
    mov   A, R6
    mov   R4, A
    mov   R7, # low  ROM_CFG_ADDR       ; ''
    mov   R6, # high ROM_CFG_ADDR       ; '?'
    acall FlashCodeEntry
    mov   _dispresult, R7
    ajmp  Dispatcher_break
;***************************************
;read config
;<b9> <> <>
FunctionB9:                             ;
    mov   Addr+0, #high ROM_CFG_ADDR    ; '?'
    mov   Addr+1, #low  ROM_CFG_ADDR    ; ''
loop_B9:                                ;while (repLen < 8)   
    mov   A, _respLen
    clr   C
    subb  A, #8
    jc    code_39E7
    ajmp  Dispatcher_break
code_39E7:                              ;
    mov   A, _respLen
    add   A, #low  ROM_CFG_ADDR         ;R7=CBYTE[ROM_CFG_ADDR+replen]
    mov   R7, A
    clr   A
    addc  A, #high ROM_CFG_ADDR         ;
    mov   DPL, R7                       ;
    mov   DPH, A                        ;
    clr   A
    movc  A, @A+DPTR                    ; 
    mov   R7, A
    mov   A, #low  Response             ;Response[respLen] = CBYTE[ROM_CFG_ADDR+replen]
    add   A, _respLen
    mov   DPL, A                        ;
    clr   A
    addc  A, #high Response
    mov   DPH, A                        ;
    mov   A, R7
    movx  @DPTR, A
    inc   _respLen                      ;
    sjmp  loop_B9
;***************************************
FunctionBB:       ;
    mov   _dispresult, #0x11
    ajmp  Dispatcher_break
;***************************************
;ReadDataFlash_V1
FunctionB7:                             ;
    mov   ROM_ADDR_H, #high DATA_FLASH_ADDR ; '' ;
code_3A10:        ;
    mov   A, _respLen
    clr   C
    subb  A, #0x80                 ; ''
    jnc   code_3A32
    mov   A, _respLen
    add   A, ACC                   ;
    mov   ROM_ADDR_L, A            ; 
    mov   ROM_CTRL, #ROM_CMD_READ  ; '' ;
    mov   A, # low  Response       ; 'L'
    add   A, _respLen
    mov   DPL, A                   ;
    clr   A
    addc  A, # high Response
    mov   DPH, A                   ;
    mov   A, ROM_DATA_L            ;
    movx  @DPTR, A
    inc   _respLen
    sjmp  code_3A10
code_3A32:                              ;
    setb  bResponsValid
    sjmp  Dispatcher_break
;***************************************
;write_dataflashV1
;<B6> <len>, <addrL> <addrh> <data...>
FunctionB6:       ;
    mov   R0, #cmdbuffer+2 ;
    mov   A, @R0
    mov   _FlAddrH, #0
    mov   _FlAddrL, A
    mov   ROM_ADDR_H, #high DATA_FLASH_ADDR ; '' ;
    clr   A
    mov   _j, A
loop1_b6:
    clr   A
    mov   _i, A
loop2_b6:
    mov   A, _j
    add   A, _FlAddrL
    mov   R7, A
    mov   A, #scramble+0
    add   A, _i
    mov   R0, A
    mov   A, @R0
    mov   R6, A
    mov   A, #cmdbuffer+4 ; '%'
    add   A, _j
    mov   R0, A
    mov   A, @R0
    xrl   A, R6
    mov   R5, A
    acall FlashData
    mov   _dispresult, R7
    mov   A, _dispresult
    jnz   Dispatcher_break
    inc   _j
    inc   _i
    cjne  _i, #4, loop2_b6
    mov   A, _j
    clr   C
    mov   R0, #cmdbuffer+1 ; '"'
    subb  A, @R0
    jc    loop1_b6
    sjmp  Dispatcher_break
;***************************************
;EraseDataFlash_V1
FunctionB5:       ;
    mov   R0, #cmdbuffer+2 ; '#'
    mov   A, @R0
    mov   _i, A
    mov   ROM_ADDR_H, # high DATA_FLASH_ADDR ; '' ;
code_3A79:        ;
    mov   A, _i
    clr   C
    subb  A, #0x80 ; ''
    jnc   Dispatcher_break
    xch   A, R7
    mov   A, _i
    xch   A, R7
    mov   R5, #0xFF
    acall FlashData
    mov   _dispresult, R7
    mov   A, _dispresult
    jnz   Dispatcher_break
    inc   _i
    sjmp  code_3A79
;***************************************
;run_cmd_v1
;<A5> <x> <runL> <runH>
FunctionA5:       ;
    mov   R0, #cmdbuffer+3              ; '$'
    mov   A, @R0
    dec   R0
    orl   A, @R0
    jz    Dispatcher_break
    setb  bRunMode
    sjmp  Dispatcher_break
;***************************************
DispatcherDefault:                      ;
    mov   _dispresult, #0xFE             ; ''
;***************************************
Dispatcher_break:                       ;
    acall ToggleProgMode
    jb    bSerialMode, code_3ADC
    mov   A, _respLen                    ;if (respLen)
    jz    code_3AB9                     ;
    setb  C
    subb  A, #0x40                      ;  result = (responseLen > 0x40) ? 0x40 : responseLen
    jc    code_3AB1
    mov   R7, #0x40                     ;
    sjmp  code_3AB3
code_3AB1:                              ;
    mov   R7, _respLen
code_3AB3:                              ;
    mov   UEP2_T_LEN, R7                ;   UEP2_T_LEN= result;
    anl   UEP2_CTRL, #not MASK_UEP_T_RES;
    ret
code_3AB9:                              ;
    mov   DPTR, #Response               ; 'L'
    mov   A, _dispresult
    movx  @DPTR, A
    clr   A
    inc   DPTR
    movx  @DPTR, A
    mov   UEP2_T_LEN, #2                ;
    anl   UEP2_CTRL, #not MASK_UEP_T_RES;
    jnb   bRunMode, DispatcherDone
    mov   _dispresult, #0x32             ; '2'
code_3ACE:                              ;
    mov   R7, _dispresult
    dec   _dispresult
    mov   A, R7
    jz    DispatcherDone
    mov   R7, # 100                     ;
    acall Shortdelay
    sjmp  code_3ACE
code_3ADC:                              ;
    mov   A, _respLen
    jz    code_3AFA
    clr   A
    mov   _i, A
code_3AE2:                              ;
    mov   A, _i
    clr   C
    subb  A, _respLen
    jnc   DispatcherDone
    mov   A, #low  Response             ; 'L'
    add   A, _i
    mov   DPL, A                        ;
    clr   A
    addc  A, #high Response
    mov   DPH, A                        ;
    movx  A, @DPTR
    mov   R7, A
    acall Serial1PutChar                ;
    inc   _i
    sjmp  code_3AE2
code_3AFA:        ;
    mov   R7, _dispresult
    acall Serial1PutChar
    clr   A
    mov   R7, A
    acall Serial1PutChar
DispatcherDone:                         ;B04
    ret

; =============== S U B R O U T I N E =======================================
HandleUsbEvents:                        ;
    jb    UIF_TRANSFER, code_3B0B       ;
    ajmp  UsbReset
code_3B0B:                              ;
    mov   A, USB_INT_ST                 ;
    anl   A, #0x3F
    jnz   code_3B14
    ajmp  EP0OutTransfer
code_3B14:                              ;
    add   A, #0xE0                      ; ''
    jnz   code_3B1B
    ajmp  EP0InTransfer
code_3B1B:                              ;
    add   A, #0xFE                      ; ''
    jz    EP2InTransfer
    add   A, #0xF2                      ; ''
    jz    EP0Setup
    add   A, #0x2E                      ; '.'
    jz    EP2OutTransfer
    ajmp  break

;copy the bulkpacket to the cmdbuffer
EP2OutTransfer:                         ;
    jb    U_TOG_OK, code_3B30           ;
    ajmp  break
code_3B30:                              ;
    mov   rLen, USB_RX_LEN              ;
    mov   RAM_1B+0, #1
    mov   RAM_1B+1, #high EP2_BUFFER
    mov   RAM_1B+2, #low  EP2_BUFFER
    mov   RAM_1E, rLen
    mov   R3, #0
    mov   R2, #0
    mov   R1, #cmdbuffer+0              ; '!'
    acall _memcpy
    clr   bSerialMode
    acall Functiondispatcher
    ajmp  break
;return the response back to the host
EP2InTransfer:                          ;
    jnb   bResponsValid, code_3B73
    clr   bResponsValid
    mov   RAM_1B+0, #1                  ;
    mov   RAM_1B+1, #high ExtraBuffer
    mov   RAM_1B+2, #low  ExtraBuffer   ;
    mov   RAM_1E,   # 0x40              ;
    mov   R3, #1                        ;xdata
    mov   R2, #high Response
    mov   R1, #low  Response            ; 'L'
    acall _memcpy
    mov   UEP2_T_LEN, #0x40             ; '@' ;
    anl   UEP2_CTRL, #not MASK_UEP_T_RES; 0xFC
    ajmp  break

code_3B73:                              ;
    mov   A, UEP2_CTRL                  ;
    anl   A, #not MASK_UEP_T_RES        ;0xFC
    orl   A, #UEP_T_RES_NAK
    mov   UEP2_CTRL, A                  ;
    ajmp  break

EP0Setup:                               ;
    ANL   UEP0_CTRL,#0F2H               ;bugfix
    mov   A, USB_RX_LEN                 ;
    xrl   A, #8
    jnz   code_3BF3
    mov   DPTR, #6
    movx  A, @DPTR
    mov   reqLen, A
    clr   bRequestError
    mov   DPTR, #0
    movx  A, @DPTR
    anl   A, #0x60
    jz    code_3B98
    setb  bRequestError
    sjmp  Setup_break

code_3B98:                              ;
    mov   DPTR, #1
    movx  A, @DPTR
    mov   bRequest, A
    mov   A, bRequest
    add   A, #0xFB                      ; ''
    jz    Set_Address
    add   A, #0xFD                      ; ''
    jz    Get_Config
    dec   A
    jz    Set_Config
    add   A, #3
    jnz   Setup_default
    mov   DPTR, #3
    movx  A, @DPTR
    add   A, #0xFE                      ;
    jz    GetDescConfig
    inc   A
    jnz   GetDescUnknown
GetDescDevice:
    mov   pDesc+0, #0xFF
    mov   pDesc+1, #high DeviceDesc
    mov   pDesc+2, #low  DeviceDesc
    mov   rLen,    #18
    sjmp  Setup_break
GetDescConfig:
    mov   pDesc+0, #0xFF
    mov   pDesc+1, #high ConfigDesc
    mov   pDesc+2, #low  ConfigDesc
    mov   rLen,    #32
    sjmp  Setup_break
GetDescUnknown:
    setb  bRequestError
    sjmp  Setup_break
Set_Address:
    sjmp  Set_Config
Get_Config:
    mov   DPTR, #0
    mov   A, wValueLo
    movx  @DPTR, A
    mov   rLen, #1
    sjmp  Setup_break
Set_Config:                             ;
    mov   DPTR, #2
    movx  A, @DPTR
    mov   wValueLo, A
    sjmp  Setup_break
Setup_default:
    setb  bRequestError
    sjmp  Setup_break

code_3BF3:                              ;
    setb  bRequestError
Setup_break:
    jnb   bRequestError, code_3C00
    mov   bRequest,  #0xFF
    mov   UEP0_CTRL, #bUEP_R_TOG or bUEP_T_TOG or UEP_R_RES_STALL or UEP_T_RES_STALL
    sjmp  break

code_3C00:                              ;
    mov   A, reqLen
    setb  C
    subb  A, rLen
    jc    code_3C0A
    mov   reqLen, rLen
code_3C0A:                              ;
    acall PreparePacket
    mov   UEP0_CTRL, #bUEP_R_TOG or bUEP_T_TOG
    sjmp  break

EP0InTransfer:                          ;
    mov   A, bRequest
    add   A, #0xFB                      ; ''
    jz    code_3C23
    dec   A
    jnz   code_3C2D
    acall PreparePacket
    xrl   UEP0_CTRL, # bUEP_T_TOG      ;
    sjmp  break

code_3C23:                              ;
    mov   A, USB_DEV_AD                 ;
    anl   A, #0x80
    orl   A, wValueLo
    mov   USB_DEV_AD, A                 ;
    sjmp  EP0OutTransfer

code_3C2D:                              ;
    clr   A
    mov   UEP0_T_LEN, A                 ;
    sjmp  EP0OutTransfer

EP0OutTransfer:                         ;
    mov   UEP0_CTRL, #UEP_T_RES_NAK     ;

break:                                  ;
    clr   UIF_TRANSFER                  ;
    ret

UsbReset:                               ;
    jnb   UIF_BUS_RST, UsbSuspend       ;
    mov   UEP0_CTRL, #UEP_T_RES_NAK     ;
    mov   UEP1_CTRL, #bUEP_AUTO_TOG     ;
    mov   UEP2_CTRL, #bUEP_AUTO_TOG or UEP_T_RES_NAK
    clr   A
    mov   USB_DEV_AD, A                 ;
    clr   UIF_SUSPEND                   ;
    clr   UIF_TRANSFER                  ;
    clr   UIF_BUS_RST                   ;
    ret

UsbSuspend:                             ;3C4E
    jnb   UIF_SUSPEND,  UsbDone         ;
    clr   UIF_SUSPEND                   ;
    ret

UsbDone:                                ;3C54
    mov   USB_INT_FG, #0xFF             ;
    ret

; =============== S U B R O U T I N E =======================================
;void main(void)
Main:                                   ;3C58
    acall InitScramble                  
    mov   SAFE_MOD, #0x55               ;
    mov   SAFE_MOD, #0xAA               ;
    mov   A, CLOCK_CFG                  ;
    anl   A, #not MASK_SYS_CK_SEL       ;
    orl   A, #4                         ;12MHz
    mov   CLOCK_CFG, A                  ;
    clr   A
    mov   SAFE_MOD, A                   ;
    clr   bSerialMode                   ;
    clr   bRunMode                      ;
    setb  bWriteProtected               ;    
    clr   bResponsValid                 ; 
    clr   EA                            ; 
    clr   TR0                           ;
    clr   TF0                           ;
    clr   U1SM0                         ;
    setb  U1SMOD                        ;
    setb  U1REN                         ;
    mov   SBAUD1, #0xF3                 ;
    clr   U1TI                          ;
    clr   U1RI                          ;
    mov   A, GLOBAL_CFG                 ;if (GLOBAL_CFG & bBOOT_LOAD)
    jnb   ACC.5, code_3C91              ;
    acall IsProgButtonPressed           ;  if(IsProgButtonPressed=={1)
    mov   A, R7
    xrl   A, #1
    jnz   code_3CBC
;enable UsbCore
code_3C91:                              ;  {
    clr   A
    mov   USB_CTRL, A                   ;
    mov   UEP2_3_MOD, #bUEP2_RX_EN | bUEP2_TX_EN;
    mov   UEP0_DMA_H, #high EP0_BUFFER  ;
    mov   UEP0_DMA_L, #low  EP0_BUFFER  ;
    mov   UEP2_DMA_H, #high EP2_BUFFER  ;
    mov   UEP2_DMA_L, #low  EP2_BUFFER  ;
    mov   UEP0_CTRL,  #UEP_T_RES_NAK    ;
    mov   UEP2_CTRL,  #bUEP_AUTO_TOG or UEP_T_RES_NAK;
    mov   USB_DEV_AD, A                 ;
    mov   UDEV_CTRL,  #bUD_PD_DIS       ;
    mov   USB_CTRL,   #bUC_DEV_PU_EN or bUC_INT_BUSY or bUC_DMA_EN;
    orl   UDEV_CTRL,  #bUD_PORT_EN      ;
    mov   USB_INT_FG, #0xFF             ;
    mov   USB_INT_EN, #7                ;
    sjmp  code_3CC9

code_3CBC:                              ;  else
    orl   TMOD, #1                      ;
    mov   TL0,  #low  0x64C0            ; 40 ms @ 12 MHz
    mov   TH0,  #high 0x64C0            ; 
    clr   TF0                           ;
    setb  TR0                           ;
;FlashAppLoop
code_3CC9:                              ;
    jnb   U1RI, code_3CCF               ;  if (U1RI) HandleSerialEvents
    acall HandleSerialEvents
code_3CCF:                              ;  if(USB_INT_FG & 0x07) HandleUsbEvents
    mov   A, USB_INT_FG                 ;
    anl   A, #7
    jz    code_3CD8
    acall HandleUsbEvents
code_3CD8:                              ;
    jnb   TF0, code_3CE4                ;   if(TF0) //timeout
    clr   TF0                           ;
    clr   TR0                           ;
    jb    bSerialMode, code_3CE4        ;
    setb  bRunMode

code_3CE4:                              ;
    jnb   bRunMode, code_3CC9           ;   if (bRunMode)
    mov   DPTR, #LoaderOptions          ;     switch to run mode
    clr   A
    movc  A, @A+DPTR
    xrl   A, #0x57
    jnz   code_3D25
    inc   DPTR
    movc  A, @A+DPTR
    xrl   A, #0xA8
    jnz   code_3D25
    mov   SAFE_MOD,  # 0x55
    mov   SAFE_MOD,  # 0xAA
    mov   CLOCK_CFG, # bOSC_EN_INT or 3 ;24MHz
    mov   SAFE_MOD,  A                  ;
    mov   USB_CTRL,  # bUC_RESET_SIE or bUC_CLR_ALL
    clr   U1SMOD
    clr   U1REN
    clr   TR0
    mov   DPTR, #LoaderOptions+3
    movc  A, @A+DPTR
    mov   R6, A
    mov   DPTR, #LoaderOptions+2
    clr   A
    movc  A, @A+DPTR
    mov   R5, A
    mov   A, R5
    xch   A, R2
    mov   A, R6
    xch   A, R2
    mov   R1, A
    mov   R3, #0
    mov   pCall+0, R3
    mov   pCall+1, R2
    mov   pCall+2, A
    acall C_Icall

code_3D25:                      ;
    mov   SAFE_MOD,   #0x55
    mov   SAFE_MOD,   #0xAA
    orl   GLOBAL_CFG, #bSW_RESET        ;
    sjmp  $                             ;

;******************************;
;from startup.a51
C_C51STARTUP:                  ;
    mov   R0, #0x7F            ;
    clr   A
Idataloop:
    mov   @R0, A
    djnz  R0, Idataloop
    mov   SP, #Stack-1         ;
    ajmp  Main

; =============== S U B R O U T I N E =======================================
HandleSerialEvents:                    ;
_chk     set R6
_len     set R5
_i       set R4

    jnb   U1RI, HandleSerial_done       ;if (!U1RI) return 
    acall Serial1GetChar                ;
    mov   A, R7                         ;if (Serial1GetChar()!=0x61) return;
    xrl   A, #0x61
    jnz   HandleSerial_done
    acall Serial1GetChar                ;if (Serial1GetChar()!=0x9E) return;
    mov   A, R7
    xrl   A, #0x9E
    jnz   HandleSerial_done
    acall Serial1GetChar
    mov   R0, #cmdbuffer+0              ;cmdbuffer[0]=Serial1GetChar();
    mov   A, R7
    mov   @R0, A
    mov   _chk, A                       ;chk=cmdbuffer[0];
    acall Serial1GetChar                ;cmdbuffer[0]=Serial1GetChar();
    inc   R0                            ;  
    mov   A, R7
    mov   @R0, A
    mov   _len, A                       ;len=cmdbuffer[0]
    mov   A, @R0
    add   A, _chk                       ;chk+=cmdbuffer[0]
    mov   _chk, A
    dec   R0
    mov   A, @R0                        ;if(cmdbuffer[0]==0xA8 |)
    xrl   A, #0xA8
    jz    code_3DF0
    mov   A, @R0                        ;  | cmdbuffer[0]==0xA7
    xrl   A, #0xA7
    jz    code_3DF0
    mov   A, @R0                        ;  | cmdbuffer[0]==0xB6
    cjne  A, #0xB6, code_3DF2           ;
code_3DF0:                              ;
    inc   _len                          ;   len+=2
    inc   _len
code_3DF2:                              ;
    clr   A                             ;i=0
    mov   _i, A                         ;
SerRecieveLoop:                         ;3DF4:
    mov   A, _i                         ;while (i < len)
    xrl   A, _len
    jz    SerLoop_break                 ;{
    acall Serial1GetChar
    mov   A, #cmdbuffer+2               ;   cmdbuffer[2+i] = Serial1GetChar
    add   A, _i
    mov   R0, A
    mov   A, R7
    mov   @R0, A
    mov   A, #cmdbuffer+2               ;   chk+=cmdbuffer[2+i]
    add   A, _i
    mov   R0, A
    mov   A, @R0
    add   A, _chk
    mov   _chk, A
    inc   _i
    sjmp  SerRecieveLoop                ;}

SerLoop_break:                          ;3E0B
    acall Serial1GetChar                ;if (Serial1GetChar != chk)
    mov   A, _chk                       ;{
    xrl   A, R7
    jz    code_3E1C
    mov   R7, #0x55                     ;  Serial1PutChar(0x55)
    acall Serial1PutChar
    mov   R7, #0xAA                     ;  Serial1PutChar(0xAA)
    ajmp  Serial1PutChar                ;}
code_3E1C:                              ;
    setb  bSerialMode                   ;bSerialMode=1;
    acall Functiondispatcher            ;Functiondispatcher
HandleSerial_done:                      ;3E21
    ret
; =============== S U B R O U T I N E =======================================
_loadPtr:                               ;
    cjne  R3, #1, code_3E2B
    mov   DPL, R1                       ;
    mov   DPH, R2                       ;
    movx  A, @DPTR
    ret
code_3E2B:                              ;
    jnc   code_3E2F
    mov   A, @R1
    ret
code_3E2F:                              ;
    cjne  R3, #0xFE, code_3E34          ; ''
    movx  A, @R1
    ret
code_3E34:                              ;
    mov   DPL, R1                       ;
    mov   DPH, R2                       ;
    clr   A
    movc  A, @A+DPTR
    ret
; =============== S U B R O U T I N E =======================================
_storePtr:                             ;
    cjne  R3, #1, code_3E44
    mov   DPL, R1                      ;
    mov   DPH, R2                      ;
    movx  @DPTR, A
    ret
code_3E44:                             ;
    jnc   code_3E48
    mov   @R1, A
    ret
code_3E48:                             ;
    cjne  R3, #0xFE, code_3E4C         ;
    movx  @R1, A
code_3E4C:                             ;
    ret
; =============== S U B R O U T I N E =======================================
_cc_case:      ;
    pop   DPH   ; Data Pointer, High Byte
    pop   DPL   ; Data Pointer, Low Byte
    mov   R0, A
code_3E52:        ;
    clr   A
    movc  A, @A+DPTR
    jnz   code_3E68
    mov   A, #1
    movc  A, @A+DPTR
    jnz   code_3E68
    inc   DPTR
    inc   DPTR
code_3E5D:        ;
    movc  A, @A+DPTR
    mov   R0, A
    mov   A, #1
    movc  A, @A+DPTR
    mov   DPL, A    ; Data Pointer, Low Byte
    mov   DPH, R0   ; Data Pointer, High Byte
    clr   A
    jmp   @A+DPTR
code_3E68:        ;
    mov   A, #2
    movc  A, @A+DPTR
    xrl   A, R0
    jz    code_3E5D
    inc   DPTR
    inc   DPTR
    inc   DPTR
    sjmp  code_3E52

; =============== S U B R O U T I N E =======================================
C_icall:
    mov   DPH, R2   ; Data Pointer, High Byte
    mov   DPL, R1   ; Data Pointer, Low Byte
    clr   A
    jmp   @A+DPTR


CopyRightMsg: db "USB DBG CH559 & ISP",0 ;

DeviceDesc:
    db     0x12
    db     1
    db     0x10
    db     1
    db     0xFF
    db     0x80 ; 
    db     0x55 ; U
    db     8
    db     0x48,0x43
    db     0xE0,0x55
    db     0x00,0x02  ;V2.0  
    db     0
    db     0
    db     0
    db     1

ConfigDesc:
    db     9
    db     2
    db     32,0
    db     1
    db     1
    db     0
    db     0x80 ; 
    db     0x32 ; 2

    db     9
    db     4
    db     0
    db     0
    db     2
    db     0xFF
    db     0x80 ; 
    db     0x55 ; U
    db     0

    db     7
    db     5
    db     0x82 ; 
    db     2
    db     0x40 ; @
    db     0
    db     0

    db     7
    db     5
    db     2
    db     2
    db     0x40 ; @
    db     0
    db     0

; =============== S U B R O U T I N E =======================================
PreparePacket:        ;
    mov   A, reqLen
    jz    code_3EF8
    clr   C
    subb  A, #8
    jc    code_3ECC
    mov   R7, #8
code_3ECA:
    sjmp  code_3ECE
code_3ECC:        ;
    mov   R7, reqLen
code_3ECE:        ;
    xch   A, R6
    mov   A, R7
    xch   A, R6
    mov   RAM_1B+0, pDesc+0
    mov   RAM_1B+1, pDesc+1
    mov   RAM_1B+2, pDesc+2
    mov   RAM_1E, R6
    mov   R3, #1
    mov   R2, #0
    mov   R1, #0
    acall _memcpy
    clr   C
    mov   A, reqLen
    subb  A, R6
    mov   reqLen, A
    mov   A, R6
    add   A, pDesc+2
    mov   pDesc+2, A
    clr   A
    addc  A, pDesc+1
    mov   pDesc+1, A
    mov   UEP0_T_LEN, R6  ;
    ret
code_3EF8:                    ;
    clr   A
    mov   UEP0_T_LEN, A ;
    ret

; =============== S U B R O U T I N E =======================================
FlashCode:                            ;
    mov   A, @R0
    mov   R6, A
    dec   R0
    mov   A, @R0
    mov   R5, A
    mov   A, R6
    mov   R4, A
    mov   R6, #high 0x3F00             ; '?'

FlashCodeEntry:                        ; 3F05
    clr   C
    mov   A, R6
    subb  A, #high BOOT_LOAD_ADDR      ;< 3800?'
    jc    code_3F1C
    mov   A, R7
    subb  A, #low  LoaderOptions       ; ''
    mov   A, R6
    subb  A, #high LoaderOptions       ; '?'
    jc    code_3F32
    setb  C
    mov   A, R7
    subb  A, #low  ROM_CFG_ADDR        ; ''
    mov   A, R6
    subb  A, #high ROM_CFG_ADDR        ; '?'
    jnc   code_3F32
code_3F1C:                             ;
    mov   ROM_ADDR_H, R6               ;
    mov   ROM_ADDR_L, R7               ;
    mov   ROM_DATA_H, R4               ;
    mov   ROM_DATA_L, R5               ;
    mov   A, ROM_CTRL                  ;
    jnb   ACC.6, code_3F32             ;
    mov   ROM_CTRL, #ROM_CMD_WRITE     ;
    mov   A, ROM_CTRL                  ;
    xrl   A, #bROM_ADDR_OK
    mov   R7, A
    ret
code_3F32:                             ;
    mov R7, #bROM_ADDR_OK              ;
    ret
; =============== S U B R O U T I N E =======================================
_memcpy:                               ;
    mov   RAM_18+0, R3
    mov   RAM_18+1, R2
    mov   RAM_18+2, R1
code_3F3B:                             ;
    mov   R7, RAM_1E
    dec   RAM_1E
    mov   A, R7
    jz    code_3F68
    mov   R3, RAM_1B+0
    inc   RAM_1B+2
    mov   A, RAM_1B+2
    mov   R2, RAM_1B+1
    jnz   code_3F4E
    inc   RAM_1B+1
code_3F4E:
    dec   A
    mov   R1, A
    acall _LoadPtr
    mov   R7, A
    mov   R3, RAM_18+0
    inc   RAM_18+2
    mov   A, RAM_18+2
    mov   R2, RAM_18+1
    jnz   code_3F60
    inc   RAM_18+1
code_3F60:                             ;
    dec   A
    mov   R1, A
    mov   A, R7
    acall _storePtr
    sjmp  code_3F3B
code_3F68:                             ;
    ret
; =============== S U B R O U T I N E =======================================
IsProgButtonPressed:                   ;3F69
    clr   A
    mov   R6, A
code_3F6B:                             ;
    jb    UDP, code_3F7B               ;
    mov   R7, #1
    acall Shortdelay
    inc   R6
    mov   A, R6
    jnb   ACC.6, code_3F6B             ;
    mov   R7, #0
    ret
code_3F7B:                             ;
    mov   R7, #1
    acall Shortdelay
    jnb   UDP, code_3F86               ;
    mov   R7, #1
    ret
code_3F86:                             ;
    mov   R7, #0
    ret
; =============== S U B R O U T I N E =======================================
FlashData:                             ;
    mov   A, R7
    add   A, ACC                       ;
    mov   ROM_ADDR_L, A                ;
    mov   ROM_DATA_L, R5               ;
    mov   A, ROM_CTRL                  ;
    jnb   ACC.6, code_3F9E             ;
    mov   ROM_CTRL, #ROM_CMD_WRITE     ;
    mov   A, ROM_CTRL                  ;
    xrl   A, # bROM_ADDR_OK
    mov   R7, A
    ret
code_3F9E:
    mov   R7, # bROM_ADDR_OK ; '@'
    ret

; =============== S U B R O U T I N E =======================================
Shortdelay:                            ;3FA1
    mov   A, R7
    rrc   A
    rrc   A
    anl   A, #0x3F
    mov   R7, A
code_3FA7:                             ;
    mov   A, R7
    jz    done
    inc   SAFE_MOD                     ;
    dec   R7
    sjmp  code_3FA7
done:                                  ;
    ret

; =============== S U B R O U T I N E =======================================
ToggleProgMode:                        ;
    mov   SAFE_MOD, #0x55              ;
    mov   SAFE_MOD, #0xAA              ;
    xrl   GLOBAL_CFG, #bCODE_WE or bDATA_WE
    clr   A
    mov   SAFE_MOD, A                  ;
    ret

; =============== S U B R O U T I N E =======================================
Serial1PutChar:                        ;
    clr   U1TI                         ;
    mov   SBUF1, R7                    ;
    jnb   U1TI, $
    clr   U1TI                         ;
    ret

; =============== S U B R O U T I N E =======================================
Serial1GetChar:
    jnb   U1RI, $
    clr   U1RI                         ;
    mov   R7, SBUF1                    ;
    ret

; =============== S U B R O U T I N E =======================================
; new to init the scramble
InitScramble:
    mov   A,'T'                        ;init scramble to 'Twch'
    mov   Scramble+0,A
    mov   A,'w'
    mov   Scramble+1,A
    mov   A,'c'
    mov   Scramble+2,A
    mov   A,'h' 
    mov   Scramble+3,A
    ret

LoaderOptions equ 0x3FF4
cseg at StartAddress or  0x7EE
    dw    0xFFFF
;0x3FF4
;LoaderOptions:
;    dw    0xFFFF                       ;must contain 0x57 and 0xA8
;3FFF6                        
;    dw    0xFFFF                       ;att. this one is lsb first 

;cseg at ROM_CFG_ADDR
;    db    0
;    db    0xD2
;    db    0x65  ;undocumented byte

;    db    0x56  ;my chip serial no
;    db    0x48
;    db    0x7B
;    db    0x48
;    db    0x43

end
